/*
 * Copyright (C) 2012-2013 Red Hat, Inc.  All rights reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 *
 * Authors: Jan Synacek <jsynacek@redhat.com>
 */
#include <konkret/konkret.h>
#include "LMI_UnixDeviceFile.h"
#include "file.h"

static const CMPIBroker* _cb = NULL;

static void LMI_UnixDeviceFileInitialize()
{
}

static CMPIStatus LMI_UnixDeviceFileCleanup(
    CMPIInstanceMI* mi,
    const CMPIContext* cc,
    CMPIBoolean term)
{
    CMReturn(CMPI_RC_OK);
}

static CMPIStatus LMI_UnixDeviceFileEnumInstanceNames(
    CMPIInstanceMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop)
{
    CMReturn(CMPI_RC_ERR_NOT_SUPPORTED);
}

static CMPIStatus LMI_UnixDeviceFileEnumInstances(
    CMPIInstanceMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop,
    const char** properties)
{
    CMReturn(CMPI_RC_ERR_NOT_SUPPORTED);
}

static CMPIStatus LMI_UnixDeviceFileGetInstance(
    CMPIInstanceMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop,
    const char** properties)
{
    LMI_UnixDeviceFile lmi_file;
    const char *path;
    char tmp[16];

    LMI_UnixDeviceFile_InitFromObjectPath(&lmi_file, _cb, cop);
    path = KChars(lmi_file.Name.value);
    fill_basic(UnixDeviceFile, path, S_IFCHR | S_IFBLK, "No such device: %s");

    sprintf(tmp, "%u", sb.st_rdev);
    LMI_UnixDeviceFile_Set_DeviceId(&lmi_file, tmp);
    sprintf(tmp, "%u",  major(sb.st_rdev));
    LMI_UnixDeviceFile_Set_DeviceMajor(&lmi_file, tmp);
    sprintf(tmp, "%u", minor(sb.st_rdev));
    LMI_UnixDeviceFile_Set_DeviceMinor(&lmi_file, tmp);

    KReturnInstance(cr, lmi_file);
    CMReturn(CMPI_RC_OK);
}

static CMPIStatus LMI_UnixDeviceFileCreateInstance(
    CMPIInstanceMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop,
    const CMPIInstance* ci)
{
    CMReturn(CMPI_RC_ERR_NOT_SUPPORTED);
}

static CMPIStatus LMI_UnixDeviceFileModifyInstance(
    CMPIInstanceMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop,
    const CMPIInstance* ci,
    const char** properties)
{
    CMReturn(CMPI_RC_ERR_NOT_SUPPORTED);
}

static CMPIStatus LMI_UnixDeviceFileDeleteInstance(
    CMPIInstanceMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop)
{
    CMReturn(CMPI_RC_ERR_NOT_SUPPORTED);
}

static CMPIStatus LMI_UnixDeviceFileExecQuery(
    CMPIInstanceMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop,
    const char* lang,
    const char* query)
{
    CMReturn(CMPI_RC_ERR_NOT_SUPPORTED);
}

CMInstanceMIStub(
    LMI_UnixDeviceFile,
    LMI_UnixDeviceFile,
    _cb,
    LMI_UnixDeviceFileInitialize())

static CMPIStatus LMI_UnixDeviceFileMethodCleanup(
    CMPIMethodMI* mi,
    const CMPIContext* cc,
    CMPIBoolean term)
{
    CMReturn(CMPI_RC_OK);
}

static CMPIStatus LMI_UnixDeviceFileInvokeMethod(
    CMPIMethodMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop,
    const char* meth,
    const CMPIArgs* in,
    CMPIArgs* out)
{
    return LMI_UnixDeviceFile_DispatchMethod(
        _cb, mi, cc, cr, cop, meth, in, out);
}

CMMethodMIStub(
    LMI_UnixDeviceFile,
    LMI_UnixDeviceFile,
    _cb,
    LMI_UnixDeviceFileInitialize())

KONKRET_REGISTRATION(
    "root/cimv2",
    "LMI_UnixDeviceFile",
    "LMI_UnixDeviceFile",
    "instance method")
/* vi: set et: */
